package geniusweb.javavenv;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.junit.Test;

import geniusweb.actions.Accept;
import geniusweb.actions.Action;
import geniusweb.actions.Offer;
import geniusweb.actions.PartyId;
import geniusweb.connection.ConnectionEnd;
import geniusweb.inform.ActionDone;
import geniusweb.inform.Inform;
import geniusweb.inform.Settings;
import geniusweb.inform.YourTurn;
import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.progress.ProgressTime;
import geniusweb.references.Parameters;
import geniusweb.references.ProfileRef;
import geniusweb.references.ProtocolRef;
import geniusweb.references.Reference;
import tudelft.utilities.listener.DefaultListenable;

public class PythonPartyFactoryE2E {
	@Test
	public void test() throws IOException, InterruptedException, PythonError {
		PythonPartyFactory factory = new PythonPartyFactory(
				new File("src/test/resources/stupidparty-1.0.0.tar.gz"));
		System.out.println(factory.getCapabilites());
		factory.close();

	}

	@Test
	public void testStart()
			throws IOException, InterruptedException, PythonError {
		PythonPartyFactory factory = new PythonPartyFactory(
				new File("src/test/resources/stupidparty-1.0.0.tar.gz"));
		System.out.println("\nGetting a party instance");
		PartyProxy proxy = factory.getInstance();
		System.out.println("proxy running:!" + proxy);
		Thread.sleep(1000);
		System.out.println("proxy terminating!");
		proxy.terminate();

		factory.close();

	}

	@Test
	public void testStartConnect()
			throws IOException, InterruptedException, PythonError {
		PythonPartyFactory factory = new PythonPartyFactory(
				new File("src/test/resources/stupidparty-1.0.0.tar.gz"));
		System.out.println("\nGetting a party instance");
		PartyProxy proxy = factory.getInstance();
		proxy.connect(new MyConnEnd());

		System.out.println("proxy running:!" + proxy);
		Thread.sleep(1000);
		System.out.println("proxy terminating!");
		proxy.terminate();
		factory.close();
	}

	@Test
	public void testStartConnectAct() throws IOException, InterruptedException,
			PythonError, URISyntaxException {

		PartyId id1 = new PartyId("party1");
		ProfileRef profileref1 = new ProfileRef("profile1");
		ProtocolRef protocolref1 = new ProtocolRef("protoco1");
		ProgressTime progress1 = new ProgressTime(1000l, new Date(12345));
		Parameters parameters1 = new Parameters();
		Settings settings = new Settings(id1, profileref1, protocolref1,
				progress1, parameters1);

		PythonPartyFactory factory = new PythonPartyFactory(
				new File("src/test/resources/stupidparty-1.0.0.tar.gz"));
		System.out.println("\nGetting a party instance");
		PartyProxy proxy = factory.getInstance();
		MyConnEnd conn = new MyConnEnd();
		proxy.connect(conn);
		conn.notifyListeners(settings);
		conn.notifyListeners(new YourTurn());

		System.out.println("proxy running:!" + proxy);

		Thread.sleep(2000);

		System.out.println("received actions: " + conn.getActions());
		assertEquals(1, conn.getActions().size());

		proxy.terminate();

		factory.close();

	}

	@Test
	public void testAcceptOpponentOffer() throws IOException,
			InterruptedException, PythonError, URISyntaxException {

		PartyId id1 = new PartyId("party1");
		PartyId id2 = new PartyId("party2");
		Bid bid = new Bid("iss1", new DiscreteValue("val1"));
		ProfileRef profileref1 = new ProfileRef("profile1");
		ProtocolRef protocolref1 = new ProtocolRef("protoco1");
		ProgressTime progress1 = new ProgressTime(1000l, new Date(12345));
		Parameters parameters1 = new Parameters();
		Settings settings = new Settings(id1, profileref1, protocolref1,
				progress1, parameters1);

		PythonPartyFactory factory = new PythonPartyFactory(
				new File("src/test/resources/stupidparty-1.0.0.tar.gz"));
		PartyProxy proxy = factory.getInstance();
		MyConnEnd conn = new MyConnEnd();
		proxy.connect(conn);
		conn.notifyListeners(settings);
		conn.notifyListeners(new ActionDone(new Offer(id2, bid)));
		conn.notifyListeners(new YourTurn());

		Thread.sleep(2000);

		System.out.println("received actions: " + conn.getActions());
		assertEquals(1, conn.getActions().size());
		Action action = conn.getActions().get(0);
		assertEquals(new Accept(id1, bid), action);

		proxy.terminate();

		factory.close();

	}

}

class MyConnEnd extends DefaultListenable<Inform>
		implements ConnectionEnd<Inform, Action> {

	private List<Action> actions = new ArrayList<>();

	@Override
	public void send(Action data) throws IOException {
		actions.add(data);
	}

	@Override
	public Reference getReference() {
		return null;
	}

	@Override
	public URI getRemoteURI() {
		return null;
	}

	@Override
	public void close() {

	}

	@Override
	public Throwable getError() {
		return null;
	}

	public List<Action> getActions() {
		return actions;
	}

}
